/*
Alle Funktionen ohne Gewhr
Michael Nowak
2019-03-20
www.mino-elektronik.de
*/


#define _DMA_TFT

#include <stdint.h>
#include <string.h>

#include "dma_tft.h"
#include "stm32f7xx.h"
#include "usart3.h"
#include "dma_tft.ext"

#define BIT(x) (1<<x)

extern char schrift_6x10[];
extern char schrift_8x16[]; 
extern char cp850_8x16[];
extern char cp866_8x16[];

static uint32_t line_cnt;              // fr DMA
static uint32_t akt_bild_adresse;      // fr DMA
volatile int v_sync, bild_sperre=0;


/*
void init_t7(void)
{
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM7, ENABLE);
    RCC->APB1ENR |= RCC_APB1ENR_TIM7EN;
    TIM7->ARR = 1000-1;                                 // Teiler fr 1ms
    TIM7->PSC = SystemCoreClock/1000000/2-1;            // Vorteiler auf 1 MHz
    TIM7->CR1 = 0x1;                                    // starten
    TIM7->DIER = 0x1;                                   // und Interrupt freigeben
    
    NVIC_EnableIRQ(TIM7_IRQn);
    NVIC_SetPriority(TIM7_IRQn,0xd);                   // geringe Prioritaet
}
*/

void init_dac2(void)
{
  RCC->APB1ENR |= RCC_APB1ENR_DACEN;
  DAC->CR = 0x30000;
  DAC->DHR8R2 = 0;
}

void setze_helligkeit(int wert)
{
  DAC->DHR8R2 = wert;
} 

void schreibe_helligkeit(int temp)
{
static int tab[] = {255,245,235,225,212,200,160,120,80,40,0};
  if(temp > (sizeof(tab)/sizeof(tab[0]))) temp = 0;
  setze_helligkeit(tab[temp]);
}

static void FMC_init(void)
{
// Enable FMC clock
    RCC->AHB3ENR |= 1;

// Takte fr PortD und PortE freigeben
	RCC->AHB1ENR |= (RCC_AHB1ENR_GPIODEN | RCC_AHB1ENR_GPIOEEN);

// AF-Ausgnge an PortD fr FMC-Bus und sync-Signale
    GPIOD->MODER |= (2 << (TFT_D0_Pin * 2) |                    
                     2 << (TFT_D1_Pin * 2) |                    
                     2 << (TFT_D2_Pin * 2) |                    
                     2 << (TFT_D3_Pin * 2) |
                     2 << (TFT_WR_Pin * 2) |
                     2 << (TFT_HSYNC_Pin * 2) |
                     2 << (TFT_VSYNC_Pin * 2) |
                     1 << (TFT_DE_Pin * 2));        // normaler Ausgang  
    
// Geschwindigkeit
    GPIOD->OSPEEDR |= (FMC_SPEED << (TFT_D0_Pin * 2) |                    
                       FMC_SPEED << (TFT_D1_Pin * 2) |                    
                       FMC_SPEED << (TFT_D2_Pin * 2) |                    
                       FMC_SPEED << (TFT_D3_Pin * 2) |
                       FMC_SPEED << (TFT_WR_Pin * 2) |
                       FMC_SPEED << (TFT_HSYNC_Pin * 2) |
                       FMC_SPEED << (TFT_VSYNC_Pin * 2) |
                       FMC_SPEED << (TFT_DE_Pin * 2)); 
	
// FMA-Bus untere Bits
    GPIOD->AFR[0] |=  (AFR_FMC << (TFT_D2_Pin * 4) |                    
                       AFR_FMC << (TFT_D3_Pin * 4) |
                       AFR_FMC << (TFT_WR_Pin * 4) ); 	
// FMC-Bus obere Bits
    GPIOD->AFR[1] |=  (AFR_FMC << ((TFT_D0_Pin-8) * 4) |                    
                       AFR_FMC << ((TFT_D1_Pin-8) * 4) |                    
                       AFR_FMC << ((TFT_HSYNC_Pin-8) * 4) |
                       AFR_FMC << ((TFT_VSYNC_Pin-8) * 4));
    
// AF Ausgnge an PortE fr FMC-Bus und sync-Signale
    GPIOE->MODER |= (2 << (TFT_D4_Pin * 2) |                    
                     2 << (TFT_D5_Pin * 2)); 
    
// hchste Geschwindigkeit
    GPIOE->OSPEEDR |= (FMC_SPEED << (TFT_D4_Pin * 2) |                    
                       FMC_SPEED << (TFT_D5_Pin * 2));
// FMA-Bus untere Bits
    GPIOE->AFR[0] |=  0L | (AFR_FMC << (TFT_D4_Pin * 4)); 	
// FMC-Bus obere Bits
    GPIOE->AFR[1] |=  (AFR_FMC << ((TFT_D5_Pin-8) * 4)); 
    
//  FMC_Bank1->BTCR[0] = 0x1011;
    FMC_Bank1->BTCR[0] =  FMC_BCR1_WREN | 
                          FMC_BCR1_MWID_0 |
                          FMC_BCR1_MBKEN;
//  FMC_Bank1->BTCR[1] = 0x0ff80c08;
    FMC_Bank1->BTCR[1] =  FMC_BTR1_DATLAT_Msk | FMC_BTR1_CLKDIV_Msk |
                          FMC_BTR1_BUSTURN_3 |
                          FMC_BTR1_DATAST_3 | FMC_BTR1_DATAST_2 |   // ca. 70 Hz
                          FMC_BTR1_ADDSET_3;
}

void init_dma2_0(void)
{
        RCC->AHB1ENR   |= RCC_AHB1ENR_DMA2EN;   // DMA2 aktivieren
        DMA2->LIFCR = 0xfd;                     // alle int-flags loeschen
        DMA2_Stream0->CR = 0x490;               // 8bit-modus  Adressmodus PINC=0 , MINC = 1
        DMA2_Stream0->NDTR = H_LINE_LEN;
        akt_bild_adresse = (uint32_t)BILDSPEICHER;
        DMA2_Stream0->PAR = (uint32_t)akt_bild_adresse;
        DMA2_Stream0->M0AR = (uint32_t)(TFT_ADR + HSYNC_ADR_MASKE - THP);
        DMA2_Stream0->CR |= 0x1;
        
        NVIC_EnableIRQ(DMA2_Stream0_IRQn);
        NVIC_SetPriority(DMA2_Stream0_IRQn,PRIO_DMA);// mittlere Prioritaet
}

void DMA2_Stream0_IRQHandler(void)
{
    DMA2->LIFCR = 0xfd;                         //alle int-flags loeschen
	if(line_cnt >= V_LEN) {
		line_cnt = 0;
        v_sync = 1;
#ifdef MIT_BLINKEN        
        if(!blink_bit)
            akt_bild_adresse = (uint32_t)BILDSPEICHER;
        else
            akt_bild_adresse = (uint32_t)BILDSPEICHER + BILDBREITE * BILDHOEHE;
#else
        akt_bild_adresse = (uint32_t)BILDSPEICHER;
#endif
            
	}	
#ifdef DE_MODE	
	if(line_cnt < TVP) {
		DMA2_Stream0->M0AR = (uint32_t)(TFT_ADR);
	} else {	
		DMA2_Stream0->M0AR = (uint32_t)(TFT_ADR + DE_ADR_MASKE - THP);
	}	
#else   
	if(line_cnt < TVP) {
		DMA2_Stream0->M0AR = (uint32_t)(TFT_ADR + HSYNC_ADR_MASKE - THP);
	} else {	
		DMA2_Stream0->M0AR = (uint32_t)(TFT_ADR + HSYNC_ADR_MASKE + VSYNC_ADR_MASKE - THP);
	}
#endif    
	if(line_cnt < Y_START_POS) {
	    DMA2_Stream0->CR = 0x490;	// 8bit-modus  Adressmodus PINC=0 , MINC = 1
        DMA2_Stream0->PAR = (uint32_t)BILDSPEICHER;
	} else {
	    DMA2_Stream0->CR = 0x690;	// 8bit-modus  Adressmodus PINC =1 , MINC = 1
		DMA2_Stream0->PAR = (uint32_t)akt_bild_adresse;
		akt_bild_adresse += (BILDBREITE);
	}	
    DMA2_Stream0->NDTR = H_LINE_LEN;
    line_cnt++;
    DMA2_Stream0->CR |= 0x1;
}  
    

void beleuchtung(uint8_t ein_aus)   // Spannungswandler ein-aus schalten
{
	RCC->AHB1ENR |= RCC_AHB1ENR_GPIOAEN;
    GPIOA->MODER |= (1<< V_LED * 2);// auf Ausgang 
    set_pin(GPIOA, V_LED, ein_aus); // und Hintergrund einschalten
}    

void init_tft(void)
{
// Takte fr PortA, PortC, PortD und PortE freigeben
	RCC->AHB1ENR |= (RCC_AHB1ENR_GPIOAEN | RCC_AHB1ENR_GPIOCEN | RCC_AHB1ENR_GPIODEN | RCC_AHB1ENR_GPIOEEN);

    init_dac2();                    // fr Hintergrundbeleuchtung
	schreibe_helligkeit(0);		    // keine Helligkeit
    init_t7_touch();                // fr periodische Abfragen
    init_usart3();                  // Ein- und Ausgabe inkl. touch-Koordinaten
    tft_clear();                    // init des Bildspeichers
    __enable_irq();
    FMC_init();                     // ext. Bus aktivieren
    init_dma2_0();                  // als TFT-Controller

	wait(200);				        // verzgert einschalten, Flackern verhindern
    init_spi2();                    // SPI2 nur als Eingang
	schreibe_helligkeit(7);		    // Grundhelligkeit vorgeben
    beleuchtung(1);                 // und einschalten

    tft_str_einmitten("mino-term 480x272 F730 V1.0", 255, 0x2a, 1);
    grafik_xy_pos(0,0);
    set_zoom(2,2);
    
/*   
    GPIOB->MODER |= BIT(5*2);
    while(1) {
      set_pin(GPIOB,5,1);
      schreibe_zeichen_dyn('A',SCHWARZ);
       set_pin(GPIOB,5,0);
    }
*/    
    
}  
