/*
Alle Funktionen ohne Gewhr
Michael Nowak
2019-03-20
www.mino-elektronik.de
*/

#define SPI2_H

#include <stdint.h>
#include "stm32f7xx.h"
#include "gpio_ports.h"
#include "tft_system.h"


#define SPI2_BUFF_LEN   0x1000
#define SPI2_BUFF_MASK  (SPI2_BUFF_LEN-1)

static uint8_t spi2_rx_puffer[SPI2_BUFF_LEN];
static uint32_t spi2_schreib_zeiger,
                spi2_lese_zeiger;

uint32_t spi2_buff_cnt(void)
{
  return(spi2_schreib_zeiger - spi2_lese_zeiger);
}  

int32_t lese_spi2_puffer(void)
{
int32_t temp = -1L;                 // default bei leerem Puffer
  if(spi2_lese_zeiger != spi2_schreib_zeiger) {
    temp = spi2_rx_puffer[spi2_lese_zeiger++];
    spi2_lese_zeiger &= SPI2_BUFF_MASK;
  }
  return(temp);
}  
  

void SPI2_IRQHandler(void)
{
uint16_t temp = SPI2->DR;
uint32_t temp_zeiger;
  temp_zeiger = (spi2_schreib_zeiger + 1) & SPI2_BUFF_MASK;
  if( temp_zeiger != spi2_lese_zeiger) {
    spi2_rx_puffer[spi2_schreib_zeiger] = temp;
    spi2_schreib_zeiger = temp_zeiger;
  }  
}  

void init_spi2(void)
{
    RCC->APB1ENR |= RCC_APB1ENR_SPI2EN;     // Takt einschalten
    NVIC_EnableIRQ(SPI2_IRQn);              // Handler anpassen
    NVIC_SetPriority(SPI2_IRQn,PRIO_SPI2);  // hohe Prioritaet
    
// AF SPI2
    GPIOB->MODER |= (2 << (SPI2_CLK  * 2) |                    
                     2 << (SPI2_NSS  * 2) |                    
                     2 << (SPI2_MOSI * 2) |                    
                     2 << (SPI2_MISO * 2));
// ohne NSS-Signal = passiv
    GPIOB->PUPDR |= (1 << (SPI2_NSS  * 2)); // mit Pullup
// AF einstellen
    GPIOB->AFR[1] |=  (AFR_SPI2 << ((SPI2_CLK-8)  * 4) |                    
                       AFR_SPI2 << ((SPI2_NSS-8)  * 4) |                    
                       AFR_SPI2 << ((SPI2_MOSI-8) * 4) |                    
                       AFR_SPI2 << ((SPI2_MISO-8) * 4)); 
// RX-interrupt nach 8 Bits
    SPI2->CR2 |= SPI_CR2_RXNEIE | SPI_CR2_FRXTH;
    SPI2->CR1 |= SPI_CR1_SPE;               // SPI2-enable 
}