/*
Alle Funktionen ohne Gewhr
Michael Nowak
2019-03-20
www.mino-elektronik.de
*/

#include <stdint.h>
#include <string.h>
#include "dma_tft.h"
#include "dma_tft.ext"


// extern char schrift_6x10[];
extern char cp850_8x16[];

uint8_t *bild = (uint8_t *)(BILDSPEICHER+X_START_POS);      // eff. Bildanfang

int zoom_x = MIN_ZOOM, zoom_y = MIN_ZOOM,                   // Init-Werte 
    hoehe = SCHRIFT_HOEHE, breite = SCHRIFT_BREITE;         // dto.
static uint8_t *aktuelle_schrift = (uint8_t *)cp850_8x16;   // aktiver Zeichensatz

static uint8_t vordergrund = SCHWARZ, hintergrund = WEISS;  // default Farben
int32_t akt_x=0, akt_y=0,           // Startadresse fr neues Zeichen
        x_raster, y_raster,         // Inkrement nach Zeichenausgabe
        zeile, spalte;              // fr Textmodus, Funktion noch ungenau
void lcd_zeile1(int8_t *s);         // Textausgabe in oberster Textzeile x/y-zoom 3/4

static uint8_t esc_flag = 0, befehl[20];    // fr esc_scanner
static uint8_t touchmeldung = 1;            // Ausgabe ber USART3 erlaubt

// Stacks fr zoom, xy und Farbe 
static uint8_t  zoom_stack_ptr, zoom_stack[MAX_ZOOM_STACK+1];
static uint16_t xy_stack_ptr, xy_stack[MAX_XY_STACK+1][2];
static uint8_t  color_stack_ptr, color_stack[MAX_COLOR_STACK+1][2];

void push_zoom(void)    // zoom_x und zoom_y als unit8_t auf Stack speichern
{
uint8_t z_xy;
  z_xy = (zoom_y & 0xf) << 4 | zoom_x & 0xf;
  if(zoom_stack_ptr < MAX_ZOOM_STACK) {
    zoom_stack[zoom_stack_ptr++] = z_xy;
  }  
}

void pop_zoom(void)     // zoom_x und zoom_y vom Stack holen
{
uint16_t z_xy = 0x11;   // default 1,1 bei Unterlauf
  if(zoom_stack_ptr) {
    z_xy = zoom_stack[--zoom_stack_ptr];  // sonst ursprnglichen Wert lesen
  }
  zoom_x = z_xy & 0xf;
  z_xy >>= 4;
  zoom_y = z_xy & 0xf;
}

void clear_zoom_stack()
{
  zoom_stack_ptr = 0;
}  

void push_xy(void)    // akt_x und akt_y als unit16_t auf Stack speichern
{
  if(xy_stack_ptr < MAX_XY_STACK) {
    xy_stack[xy_stack_ptr][0] = akt_x;
    xy_stack[xy_stack_ptr][1] = akt_y;
    xy_stack_ptr++;
  }  
}

void pop_xy(void)     // zoom_x und zoom_y vom Stack holen
{
  if(xy_stack_ptr) {
    akt_x = xy_stack[xy_stack_ptr][0];
    akt_y = xy_stack[xy_stack_ptr][1];
    xy_stack_ptr--;
  }  
}

void clear_xy_stack()
{
  xy_stack_ptr = 0;
}  


void push_farbe(void)    // Vordergrundfarbe und Hintergrundfarbe auf Stack speichern
{
  if(color_stack_ptr < MAX_COLOR_STACK) {
    color_stack[color_stack_ptr][0] = vordergrund;
    color_stack[color_stack_ptr][1] = hintergrund;
    color_stack_ptr++;
  }  
}

void pop_farbe(void)     // zoom_x und zoom_y vom Stack holen
{
  if(color_stack_ptr) {
    vordergrund = color_stack[color_stack_ptr][0];
    hintergrund = color_stack[color_stack_ptr][1];
    color_stack_ptr--;
  }  
}

void clear_color_stack()
{
  color_stack_ptr = 0;
}  


void set_zoom(int x, int y)
{
  if(x < MIN_ZOOM) x = MIN_ZOOM;
  if(x > MAX_ZOOM) x = MAX_ZOOM;
  if(y < MIN_ZOOM) y = MIN_ZOOM;
  if(y > MAX_ZOOM) y = MAX_ZOOM;
  zoom_x = x;
  zoom_y = y;
	x_raster = breite * zoom_x;
	y_raster = hoehe * zoom_y;
}    

static inline void set_pixel(uint32_t x, uint32_t y, uint8_t farbe)
{
  if(x <= MAX_X && y <= MAX_Y) bild[(x + y*BILDBREITE)] = farbe;
#ifdef MIT_BLINKEN                  // zustzlich alt. Bildspeicher beschreiben
  if(!(farbe&BLINKEN)) bild[(x + y*BILDBREITE)+ BILDGROESSE] = farbe;
  else bild[(x + y*BILDBREITE)+ BILDGROESSE] = farbe;
#endif  

}

void set_xy(uint16_t x, uint16_t y)
{
  akt_x = x;
  akt_y = y;
}

void schreibe_zeichen_dyn(uint8_t c, uint8_t farbe)
{	
uint32_t zeile, spalte;
uint32_t muster, i, j;
uint32_t eff_x, eff_y;
uint8_t *adr, *temp_adr;	
uint8_t *zeichen = &aktuelle_schrift[(c+1)*ZEICHEN_HOEHE];	// Basisadresse des Zeichens
  eff_y = akt_y;
  zeile = ZEICHEN_HOEHE;
  // testen, ob Zeichen voll in den Bildbereich passt
  if(akt_x >= 0 && akt_x + ZEICHEN_LAENGE * zoom_x < MAX_X && 
     akt_y >= 0 && akt_y + ZEICHEN_HOEHE * zoom_y < MAX_Y) {
    // keine weiteren Tests beim Schreiben der Pixel notwendig       
    while(zeile--) {	        // gesamte hhe
      eff_x = akt_x;
	  muster = *zeichen++;		// daten der Zeile
	  adr = temp_adr;	
	  spalte = ZEICHEN_LAENGE;
	  while(spalte--) {
  	    temp_adr = adr = (uint8_t *) (bild + (eff_x+eff_y*BILDBREITE));
	    i= 0;
	    while(i < zoom_y) {		// zoom y
          j = 0;
	      while(j < zoom_x) {	// zoom x
            if(muster & 0x80) {
	          if(!(farbe&TRANSPARENT)) {
                *adr = farbe;   // Vordergrund zeichnen
#ifdef MIT_BLINKEN
                if(farbe&BLINKEN) *(adr+BILDGROESSE) = blink_farbe;
                else  *(adr+BILDGROESSE) = farbe;
#endif                
              }  
	        } else {
	          if(!(hintergrund&TRANSPARENT)) *adr = hintergrund; // Hintergrund zeichnen
	        }	
	        adr++;
            j++;
          } // while(j)
	      temp_adr += BILDBREITE;	
	      adr = temp_adr;
          i++;
        } // while(i)
	    muster <<= 1;	        // nchstes pixel
	    eff_x += zoom_x;
      } // while(spalte)
	  eff_y += zoom_y;	        // nchste Zeile
    } // while(zeile)
  } else {                        
    // Zeichen liegt am Bildrand; daher jedes Pixel auf gueltigen Bereich testen
    while(zeile--) {	        // gesamte hhe
      eff_x = akt_x;
	  muster = *zeichen++;		// daten der Zeile
	  spalte = ZEICHEN_LAENGE;
	  while(spalte--) {
	    i= 0;
	    while(i < zoom_y) {		// zoom y
          j = 0;
	      while(j < zoom_x) {	// zoom x
		    if(muster & 0x80) {
              if(!(farbe&TRANSPARENT)) {
                set_pixel(eff_x+j, eff_y+i,farbe);  // Vordergrund zeichnen
#ifdef MIT_BLINKEN
                if(farbe&BLINKEN) *(adr+BILDGROESSE) = blink_farbe;
                else  *(adr+BILDGROESSE) = farbe;
#endif        
              }  
		    } else {
		      if(!(hintergrund&TRANSPARENT)) set_pixel(eff_x+j, eff_y+i,hintergrund); // Hintergrund zeichnen
	        }	
            j++;
          } // while(j)
          i++;
        }   // while(i)
	    muster <<= 1;	        // nchstes pixel
	    eff_x += zoom_x;
      }     // while(spalte)
	  eff_y += zoom_y;	        // nchste Zeile
    }       // while(zeile)
  }    
}


void string_ausgabe(int8_t *s, uint8_t farbe)
{
uint8_t c;
    while(c = *s++) {
      schreibe_zeichen_dyn(c,farbe);
      akt_x += ZEICHEN_LAENGE * zoom_x;
    }
}		

void schreibe_string(int8_t *s, uint8_t farbe)
{
    zoom_x = zoom_y = 1;
    string_ausgabe(s,farbe);
}

void schreibe_string2(int8_t *s, uint8_t farbe)
{
    zoom_x = zoom_y = 2;
    string_ausgabe(s,farbe);
}

void schreibe_string3(int8_t *s, uint8_t farbe)
{
    zoom_x = zoom_y = 3;
    string_ausgabe(s,farbe);
}

void schreibe_string_mittig(int8_t *s, uint8_t farbe)
{
uint8_t c;
    c=0; while(s[c]) c++;
    akt_x = (MAX_X - c * ZEICHEN_LAENGE*zoom_x)/2;	
    while(c = *s++) {
        schreibe_zeichen_dyn(c,farbe);
	akt_x += ZEICHEN_LAENGE*zoom_x;
    }		
}

void lcd_zeile_x(int8_t *s, uint8_t farbe)
{
uint8_t c;
    push_zoom();
    zoom_x = 3;zoom_y = 4;                          // SCHN GROSS
    c=0; while(s[c]) c++;                           // Zeichen zhlen
    akt_x = (MAX_X - c * ZEICHEN_LAENGE*zoom_x)/2;	// Startpunkt links ermitteln
    fuelle_rechteck(0, akt_y, akt_x, akt_y+ZEICHEN_HOEHE*zoom_y,WEISS); // links lschen
    while(c = *s++) {
      schreibe_zeichen_dyn(c,farbe);
	  akt_x += ZEICHEN_LAENGE*zoom_x;
    }		
    fuelle_rechteck(akt_x, akt_y, MAX_X, akt_y+ZEICHEN_HOEHE*zoom_y,WEISS); // rechts lschen
    pop_zoom();
}

	
void lcd_zeile1(int8_t *s)
{
  set_xy(0,77);
  lcd_zeile_x(s,vordergrund);	
}

void lcd_zeile2(int8_t *s)
{
  set_xy(0,170);
  lcd_zeile_x(s,SCHWARZ);	
}

void tft_str_einmitten(int8_t *s, uint16_t y_pos, int8_t farbe, int8_t zoom)
{
uint16_t temp_y;
    temp_y = akt_y;
    push_zoom();
    zoom_x = zoom_y = zoom;
    akt_y = y_pos;
    schreibe_string_mittig(s, farbe);
    pop_zoom();
    akt_y = temp_y;
}


int sgn(int x){
    return (x > 0) ? 1 : (x < 0) ? -1 : 0;
}

// zeichne_linie(x1,y1,x2,y,farbe);
void zeichne_linie(int xstart, int ystart, int xend, int yend, int8_t farbe)
{
int x, y, t, dx, dy, incx, incy, pdx, pdy, ddx, ddy, es, el, err;

    dx = xend - xstart;
    dy = yend - ystart;
    incx = sgn(dx);
    incy = sgn(dy);
    if(dx<0) dx = -dx;
    if(dy<0) dy = -dy;

    if (dx>dy){
        pdx=incx; pdy=0;
        ddx=incx; ddy=incy;
        es =dy;   el =dx;
    } else {
        pdx=0;    pdy=incy;
        ddx=incx; ddy=incy;
        es =dx;   el =dy;
    }

    x = xstart;
    y = ystart;
    err = el/2;
    set_pixel(x,y, farbe);

    for(t=0; t<el; ++t) {
        err -= es;
        if(err<0) {
            err += el;
            x += ddx;
            y += ddy;
        } else {
            x += pdx;
            y += pdy;
        }
        set_pixel(x,y, farbe);
    }
}

// zeichne_hor_linie(x1,x2,y,farbe);
void horizontale_linie(int32_t x1, int32_t x2, int32_t y, uint8_t farbe)
{
uint16_t temp;
uint8_t *adr;	
    if(x1 < 0)  x1 = 0;
    if(x2 < 0)  x2 = 0;
    if(y < 0)   y = 0;
    if(x1 > MAX_X)  x1 = MAX_X;
    if(x2 > MAX_X)  x2 = MAX_X;
    if(y > MAX_Y)   y = MAX_Y;

    if(x1 > x2) {
        temp = x1;
        x1 = x2;
        x2 = temp;
    }	
    adr = (bild + (x1+y*BILDBREITE));
    while(x1 <= x2) {
      *adr = farbe;
#ifdef MIT_BLINKEN      
      if(!(farbe&BLINKEN)) *(adr + BILDGROESSE) = farbe;
      else *(adr + BILDGROESSE) = blink_farbe;
#endif      
      adr++;
      x1++;

    }
}

void zeichne_rechteck(int32_t x1, int32_t y1, int32_t x2, int32_t y2, int8_t farbe)
{
    zeichne_linie(x1, y1, x1, y2, farbe);	// links senkrecht
    zeichne_linie(x2, y1, x2, y2, farbe);	// rechts senkrecht
    zeichne_linie(x1, y1, x2, y1, farbe);	// oben
    zeichne_linie(x1, y2, x2, y2, farbe);	// unten
}

void fuelle_rechteck(int32_t x1, int32_t y1, int32_t x2, int32_t y2, int8_t farbe)
{
int16_t temp;	
    if(y1 > y2) {
        temp = y1;
        y1 = y2;
        y2 = temp;
    }
    while(y1 <= y2) {
        horizontale_linie(x1, x2, y1, farbe);
        y1++;
    }
}	

void loesche_tft(uint8_t farbe)
{
    fuelle_rechteck(0,0,MAX_X,MAX_Y,farbe);
}

void schreibe_zeichen(unsigned char c, char farbe)
{
    zoom_y = zoom_x;
    schreibe_zeichen_dyn(c, farbe);
    
}	


void tft_clear(void)
{
fuelle_rechteck(0,0,MAX_X,MAX_Y,(hintergrund));
	akt_y = 0;
	akt_y = 0;
	zeile = 0;
	spalte = 0;
	x_raster = breite * zoom_x;
	y_raster = hoehe * zoom_y;
}


void copy_line(int dest, int src)
{
static uint8_t *adr_dest, *adr_src;
int n;	
	if(dest > MAX_Y || src > MAX_Y) return;
	if(dest < 0 || src < 0) return;
    adr_src = (bild + (src * BILDBREITE));
    adr_dest = (bild + (dest * BILDBREITE));
	for(n = 0; n < H_LINE_LEN; n++)
	  *(uint8_t *)adr_dest++ = *(uint8_t *)adr_src++;
#ifdef MIT_BLINKEN
    adr_src = (bild + (src * BILDBREITE) + BILDGROESSE);
    adr_dest = (bild + (dest * BILDBREITE) + BILDGROESSE);
	for(n = 0; n < H_LINE_LEN; n++)
	  *(uint8_t *)adr_dest++ = *(uint8_t *)adr_src++;

#endif    
}


void scroll(void)
{
static short x, y0, y_diff;

	y_diff = y_raster;
	for(y0 = 0; y0 <= MAX_Y-y_diff; y0++) {
	  copy_line(y0, y0+y_diff);
	}
	while(y0 <= MAX_Y) {
		for(x=0; x <= MAX_X; x++)
			set_pixel(x,y0,hintergrund);
		y0++;
	}
}


void setze_zeichensatz(char temp)
{
    switch(temp) {
      case 1:
		aktuelle_schrift = (uint8_t *)cp850_8x16;
        break;
      case 2:
//		aktuelle_schrift = (uint8_t *)schrift_6x10;
        break;
      case 3:
//		aktuelle_schrift = (uint8_t *)cp866_8x16;
        break;
      case 4:
//		aktuelle_schrift = (uint8_t *)schrift_8x16;
        break;
      default:
		aktuelle_schrift = (uint8_t *)cp850_8x16;
        break;
	} 
    hoehe = ZEICHEN_HOEHE;
    breite = ZEICHEN_LAENGE;
	x_raster = breite * zoom_x;
	y_raster = hoehe * zoom_y;
}			 
	 
void schreibe_c(char c)                 // Ausgabe fr Text
{
	schreibe_zeichen(c, vordergrund);
	akt_x += x_raster;
	if(akt_x > MAX_X) {
		akt_x = 0;
		akt_y += y_raster;
		if(akt_y + y_raster > MAX_Y+1) {
			scroll();
			akt_y -= y_raster;
		}	 
	}
}

void esc_scanner(char c)                // Verarbeitung von ser. Befehlen
{	 
static uint8_t count, bef_len;
short x1=0, y1=0, x2=0, y2=0;

uint8_t temp = 0;
//	printf("%2x  ",c);
	if(count == 0) {
		switch(c) {
			case ESC:
				bef_len = 0;
				to_outbuf3('*');
				esc_flag = 0;
				break;
			case 'c':
				bef_len = 3;
				break;
			case 'g':
				bef_len = 11;
				break;
			case 'G':
				bef_len = 7;
				break;
			case 'C':
				bef_len = 3;
				break;
			case 'V':
				printf("mino-term F730 V1.0;%d;%d;%d\0", ALTERNATIVE_SEITEN, MAX_X+1, MAX_Y+1);
				esc_flag = 0;
				break;
		}	
	}
	if(bef_len == 0) return;	// keine weitere Zeichen
	befehl[count++] = c;
	if(count >= bef_len) {		// interpretieren
		switch (*befehl) {
			case 'C':
				zeile = befehl[1] - 0x20;
				spalte = befehl[2] - 0x20;
				if(zeile < 0) zeile = 0;
				if(spalte < 0) spalte = 0;
				x_raster = breite * zoom_x;
				y_raster = hoehe * zoom_y;
				akt_x = spalte * x_raster;
				akt_y = zeile * y_raster;
				break;
			case 'c':
				temp = befehl[2];
				switch(befehl[1]) {
						
					case '3':
						vordergrund = (temp);
						break;
					case '4':
						hintergrund = (temp);
						break;
					case '5':
						temp &=0x0f;		// begrenzen
						if(temp > MAX_ZOOM) temp = MAX_ZOOM;
						if(temp < MIN_ZOOM) temp = MIN_ZOOM;
						zoom_x = zoom_y = temp;;
						x_raster = breite * zoom_x;
						y_raster = hoehe * zoom_y;
						break;
					case '6':			// zeichensatz
						setze_zeichensatz(temp);
						break;
					case '7':
						schreibe_blinkfrequenz(temp);	
						break;
					case '8':
						schreibe_helligkeit(temp);	
						break;
					case 'b':	
						switch(temp) {
							case 0: set_baudrate3(9600);break;
							case 1: set_baudrate3(19200);break;
							case 2: set_baudrate3(38400);break;
							case 3: set_baudrate3(57600);break;
							case 4: set_baudrate3(115200);break;
							case 5: set_baudrate3(230400);break;
							case 6: set_baudrate3(250000);break;
							case 7: set_baudrate3(312500);break;
							case 8: set_baudrate3(500000);break;
							case 9: set_baudrate3(625000);break;
						}	
						break;
					case 'O':
						tft_alarm(temp*10);
						break;
					case 't':
						if(temp) touchmeldung = 1;
						else touchmeldung = 0;
						break;
#ifdef MIT_BLINKEN                        
					case '1':
						blink_farbe = (temp);
						break;
#endif                        
				}
				break;
			case 'g':
				x1 = (befehl[2]<<8) + befehl[3];
				y1 = (befehl[4]<<8) + befehl[5];
				x2 = (befehl[6]<<8) + befehl[7];
				y2 = (befehl[8]<<8) + befehl[9];
				temp = befehl[10];
				switch(befehl[1]) {
					case 'l':
						zeichne_linie(x1,y1,x2,y2,(temp));
						break;
					case 'r':
						zeichne_rechteck(x1,y1,x2,y2,(temp));
						break;
					case 'f':
						fuelle_rechteck(x1,y1,x2,y2,(temp));
						break;
				}
				break;
			case 'G':
				x1 = (befehl[2]<<8) + befehl[3];
				y1 = (befehl[4]<<8) + befehl[5];
				temp = befehl[6];
				switch (befehl[1]) {
					case 'p':
						set_pixel(x1,y1,(temp));
						break;
					case 'c':
                        push_xy();      // Cursor fr Zeichenausgabe nicht verndern
						akt_x = x1;
						akt_y = y1;
						schreibe_zeichen(temp, vordergrund);
						pop_xy();
						break;
					case 't':
						akt_x = x1;
						akt_y = y1;
						x_raster = temp;
						break;
				}
				break;
		}
		esc_flag = 0;
		count = 0;
		bef_len = 0;
	}
}

static void sende_touch()
{
static uint8_t temp;
int t_x, t_y, t_aktiv;
    lese_touch(&t_x, &t_y);
	t_aktiv = touch_aktiv;
	touch_aktiv = 0;			    // nur einmal ausgeben
	if(!touchmeldung) return;	    // nicht zulaessig
    
   
	temp = ((t_x >> 7) & 0x3f) | 0x80;
	if(t_aktiv == 1) temp |= 0x40;	// Kennung fr tastendruck
	to_outbuf3(temp);				// obere 5 bits von x
	temp = (t_x & 0x7f);		    // restliche 7 bit
	to_outbuf3(temp);				// obere 5 bits von x
	temp = ((t_y >> 7) & 0x3f);	    // obere 5 bits von y
	to_outbuf3(temp);				// obere 5 bits von x
	temp = (t_y & 0x7f);		    // restliche 7 bit
	to_outbuf3(temp);				// obere 5 bits von x
}	


void putstr(char *p)
{ 
  while(*p) schreibe_c(*p++);
}

void putstrn(char *p, char n)
{
  while(n--) schreibe_c(*p++);
}


void tft_fcol(unsigned char fcol)
{ vordergrund = fcol;}

void tft_bcol(unsigned char bcol)
{ hintergrund = bcol;}

void tft_gchar(short x1,short y1, uint8_t z)
{
  push_xy();        // Cursor fr Zeichenausgabe nicht verndern
  akt_x = x1;
  akt_y = y1;
  schreibe_zeichen(z, vordergrund);
  pop_xy();
}


void tft_settextcursor(short x1,short y1, uint8_t abstand)
{
  akt_x = x1;
  akt_y = y1;
  x_raster = abstand;
}

void tft_gtext(short x, short y, short xraster , char *p)
{
   while(*p){
      tft_gchar(x, y, *p++);
      x += xraster;
   }
}

void grafik_xy_pos(int x_pos, int y_pos)
{
  if(x_pos < 0) y_pos = 0;
  if(y_pos < 0) y_pos = 0;
  x_raster = breite * zoom_x;
  y_raster = hoehe * zoom_y;
  akt_x = x_pos;
  akt_y = y_pos;
}

void blanks_entfernen(char *s)
{
char temp,i;
  temp = strlen(s)-1;
  while(temp && (s[temp] == ' ')) s[temp--] = 0; /* blanks am ende loeschen */
  temp = 0;
  while(s[temp] == ' ') temp++;		/* blanks am anfang suchen */
  if(temp) {				/* nur, wenn welche vorhanden */
    i=0;
    do {
      s[i++] = s[temp];			/* verschieben */
    } while(s[temp++]);			/* bis ende erreicht */
  }
}

void tft_gr_cursor_on(short offset, char on_farbe)
{
  fuelle_rechteck(akt_x+offset,
	    akt_y+SCHRIFT_HOEHE*zoom_y-2,
	    akt_x+offset+SCHRIFT_BREITE*zoom_x,
	    akt_y+SCHRIFT_HOEHE*zoom_y-3,
	    on_farbe);
  
}

void tft_gr_cursor_off(short offset, char off_farbe)
{
  
  fuelle_rechteck(akt_x+offset,
	    akt_y+SCHRIFT_HOEHE*zoom_y-2,
	    akt_x+offset+SCHRIFT_BREITE*zoom_x,
	    akt_y+SCHRIFT_HOEHE*zoom_y-3,
	    off_farbe);
  
}

void hauptprogramm()
{
uint8_t c;
    init_tft();
	while(1) {
		do {
 			if(touch_aktiv) {
              sende_touch();
			}
		} while(!read_inbuf3_count() && !spi2_buff_cnt());
        
        if(spi2_buff_cnt())
          c = lese_spi2_puffer();
        else
		  c = read_inbuf3();
		if(esc_flag) {
			esc_scanner(c);
			c=0;
		} else {                            // Steuerzeichen
			switch(c) {
                case GLOCKE:
                    tft_alarm(300);         // Piepsen
                break;    
				case CLEAR:			        // clear + home_up
					tft_clear();
				case HOME:			        // home-up						
					akt_x = 0;
					akt_y = 0;
				break;
				case LF:                    // Zeilenvorschub
					akt_y += y_raster;
					if(akt_y + y_raster > MAX_Y+1) {
						scroll();	
						akt_y -= y_raster;	// y nicht erhhen
					}
					break;
				case RTN:
					akt_x = 0;              // an Zeilenanfang
					break;
				case ESC:                   // neuer Befehl
					esc_flag = 1;
					break;
				default:
					schreibe_c(c);
			}		
		}
    }
}
