/*
Alle Funktionen ohne Gewhr
Michael Nowak
2019-03-20
www.mino-elektronik.de
*/

#define _USART3

#include "stm32f730xx.h"
#include "usart3.h"


static uint8_t usart3_rx_puffer[INBUF3_LEN];
static uint8_t usart3_tx_puffer[OUTBUF3_LEN];
static uint32_t usart3_rx_schreib_zeiger,
                usart3_rx_lese_zeiger;
static uint32_t usart3_tx_schreib_zeiger,
                usart3_tx_lese_zeiger;
static uint8_t ser_status;

char set_baudrate3(int baud)
{
  USART3->BRR = APB1_TAKT/baud;
  return(0);
}

void USART3_IRQHandler(void)
{
uint32_t temp, sr_temp;
uint32_t temp_zeiger;
    sr_temp = USART3->ISR;
    if(sr_temp & USART_ISR_TXE) {       // Tx-Flag testen
        if(usart3_tx_schreib_zeiger != usart3_tx_lese_zeiger) { // noch zeichen im puffer
	      USART3->TDR = usart3_tx_puffer[usart3_tx_lese_zeiger++];	// Zeichen ausgeben
          usart3_tx_lese_zeiger &= OUTBUF3_MASK;
        } else  {
          USART3->CR1 &= ~USART_CR1_TXEIE;  // sonst weitere Interrupts sperren */
          __ISB();                          // Abwarten
        }  
    }
    if(sr_temp & USART_ISR_RXNE) {      // Rx-Flag testen
        temp = USART3->RDR;             // Zeichen lesen
        USART3->ICR = USART_ICR_ORECF;  // OVE-Bit immer loeschen
        temp_zeiger = (usart3_rx_schreib_zeiger + 1) & INBUF3_MASK;
        if( temp_zeiger != usart3_rx_lese_zeiger) {
          usart3_rx_puffer[usart3_rx_schreib_zeiger] = temp;
          usart3_rx_schreib_zeiger = temp_zeiger;
        }
    }
    if(sr_temp & USART_ISR_ORE) {       // Ueberlauf-Flag testen
        USART3->ICR = USART_ICR_ORECF;  // OVE-Bit immer loeschen
    }
 }

void clr_outbuf3()
{
//  USART_ITConfig(USART3, USART_IT_TXE, DISABLE);
  USART3->CR1 &= ~(USART_CR1_TXEIE); // TXIE abschalten
  __ISB();
  usart3_tx_lese_zeiger = 0;
  usart3_tx_schreib_zeiger = 0;
}

void clr_inbuf3()
{
//    USART_ITConfig(USART3, USART_IT_RXNE, DISABLE);
  USART3->CR1 &= ~(USART_CR1_RXNEIE); // RXIE abschalten
  __ISB();
  usart3_rx_lese_zeiger = 0;
  usart3_rx_schreib_zeiger = 0;
//  USART_ITConfig(USART3, USART_IT_RXNE, ENABLE);
  USART3->CR1 |= (USART_CR1_RXNEIE); // RXIE einschalten
}

void init_usart3(void)
{
	RCC->AHB1ENR |= RCC_AHB1ENR_GPIOBEN;
    RCC->APB1ENR |= RCC_APB1ENR_USART3EN;
// AF fr USART3    
    GPIOB->MODER |=  (2 << (TxD3_Pin * 2) | 2 << (RxD3_Pin * 2));

// USART3 Alternative-Funktions mit den IO-Pins verbinden
    GPIOB->AFR[1] |=  (7 << ((TxD3_Pin-8) * 4) | 7 << ((RxD3_Pin-8) * 4));

// alle weiteren Einstellungen default 8N1 
    set_baudrate3(DEF_BAUDRATE);
  
// enable UART Interrupt-Vector
  NVIC_SetPriority(USART3_IRQn,PRIO_USART3);
  NVIC_EnableIRQ(USART3_IRQn);                         // USART3 aktivieren
    
  clr_inbuf3();
  clr_outbuf3();
// RXIE und Rx + Tx einschalten  
  USART3->CR1 |= USART_CR1_RXNEIE | USART_CR1_UE | USART_CR1_RE | USART_CR1_TE;
  ser_status = 1;       // ist initialisiert
}	

void to_outbuf3(char c)
{
uint32_t temp_zeiger;
  if(!ser_status) init_usart3() ;
  do {
    temp_zeiger = (usart3_tx_schreib_zeiger + 1) & OUTBUF3_MASK;
  } while( temp_zeiger == usart3_tx_lese_zeiger);    // warten, bis wieder platz ist
  usart3_tx_puffer[usart3_tx_schreib_zeiger] = c;
  usart3_tx_schreib_zeiger = temp_zeiger;
  USART3->CR1 |= USART_CR1_TXEIE; // per Interrupt ausgeben
  
}


char read_inbuf3()
{
int32_t temp = 0;                 // default bei leerem Puffer

  if(!ser_status) init_usart3() ;
  if(usart3_rx_lese_zeiger != usart3_rx_schreib_zeiger) {
    temp = usart3_rx_puffer[usart3_rx_lese_zeiger++];
    usart3_rx_lese_zeiger &= INBUF3_MASK;
  }
  return(temp);
}

uint32_t read_inbuf3_count()
{
	return(usart3_rx_schreib_zeiger - usart3_rx_lese_zeiger);
}	

void sende_string(char *s)
{
//  while(outbuf3_anzahl);		/* warten, wenn puffer voll ist */
  while(*s) to_outbuf3(*(s++));
//  putchar(10);
} 

int32_t putchar(int32_t x)
{
  if((uint8_t)x == 10) to_outbuf3(13); 
  to_outbuf3((uint8_t)x);
  return(x);
}